.icons-on .contenttype-contentcategoryconfiguration,
.icons-on .contenttype-contentcategorygroup,
.icons-on .contenttype-contentcategory {
    background: transparent url("../contenttypes-sprite.png") no-repeat scroll 0 -644px;
}
.icons-on .contenttype-contentcategoryconfiguration,
.icons-on .contenttype-contentcategorygroup,
.icons-on .contenttype-contentcategory {
    display: inline-block;
    line-height: 2em;
    padding-left: 20px !important;
}

#content div.categorized-elements {
    display: inline-block;
}

#content div.categorized-elements h2 {
    margin-bottom: 0.3em;
}

/* display number of elements smaller */
.tooltip.deactivated.tooltipstered > span {
    font-size: 80%;
}

#content div.categorized-elements a:link,
#content div.categorized-elements a:hover,
#content div.categorized-elements a:visited {
    color: #666;
    /* added !important to override default behavior of base.css that removes border-bottom in table.listing a */
    border: 1px solid #ccc !important;
    background: #EDEDED;
    padding-left: 0.2em;
    padding-right: 0.2em;
    padding-top: 0.1em;
    padding-bottom: 0.3em;
    border-radius: 5px;
    /* force height and display for Chrome */
    height: 16px;
    display: inherit;
}

span.warn_filesize {
    color: red;
    cursor: help;
    font-weight: bold;
}

img.categorized_elements_more_infos_icon {
    padding-right: 0.2em;
}

div.tooltipster-content li {
    font-size: 0.9em;
    display: block;
    margin: 0.2em 0;
}

#content table.iconified-listing {
    width: 100%;
}

#content .iconified-listing td.iconified-print a,
#content .iconified-listing td.iconified-confidential a {
    font-family: 'FontAwesome';
    font-size: 120%;
    cursor: help;
}

/* need to use !important to have a border-bottom because it is removed using !important in base.css... */
#content .iconified-listing td.iconified-print a.editable,
#content .iconified-listing td.iconified-confidential a.editable {
    border-bottom: 1px dotted !important;
    padding-bottom: 2px;
    cursor: pointer;
}

#content .iconified-listing td.iconified-print a,
#content .iconified-listing td.iconified-confidential a.active {
    color: #EA0000;
}

#content .iconified-listing td.iconified-print a.active,
#content .iconified-listing td.iconified-confidential a {
    color: #75ad0a;
}

#content .iconified-listing td.iconified-print a.active:hover,
#content .iconified-listing td.iconified-confidential a:hover {
    color: #0B8400!important;
}

#content .iconified-listing td.iconified-print a:hover,
#content .iconified-listing td.iconified-confidential a.active:hover {
    color: #B50000!important;
}

#content .iconified-listing td.iconified-print a.deactivated,
#content .iconified-listing td.iconified-confidential a.deactivated {
    color: #999;
}

#content .iconified-listing td.iconified-print a.deactivated:hover,
#content .iconified-listing td.iconified-confidential a.deactivated:hover {
    color: #666!important;
}

#content .iconified-listing td.iconified-print a.error,
#content .iconified-listing td.iconified-confidential a.error {
    color: #E58200;
}

#content .iconified-listing td.iconified-print a.error:hover,
#content .iconified-listing td.iconified-confidential a.error:hover {
    color: #C97200!important;
}

#content .iconified-listing td.iconified-print a:before {
    content: "\f02f";
}

#content .iconified-listing td.iconified-confidential a:before {
    content: "\f09c";
}

#content .iconified-listing td.iconified-confidential a.active:before {
    content: "\f023";
}

/* reduce select2 font-size */
.select2-container, .select2-results {
    font-size: 90%;
}
/* add left margin to subcategories displayed in the select2 dropdown */
ul.select2-results span.subcategory {
    margin-left: 1em;
}
/* reduce space under selectable elements displayed in the dropdown */
ul.select2-results .select2-result-label {
    padding-bottom: 0px;
}
